use std::{os::unix::io::{AsRawFd, RawFd}, process::Child};

fn pidfd_open(pid: RawFd) -> nix::Result<PidFd> {
    // SAFETY: passing arguments as specified for pidfd_open()
    match unsafe { libc::syscall(libc::SYS_pidfd_open, pid as libc::pid_t, 0 as libc::c_uint) } {
        fd if fd >= 0 =>  Ok(PidFd(fd as RawFd)),
        _ => {
            // SAFETY: libc must be sane
            let errno_location = unsafe { libc::__errno_location() };
            if errno_location.is_null() {
                Err(nix::errno::Errno::UnknownErrno)
            } else {
                // SAFETY: pointer is checked for null; libc must be sane
                let errno = unsafe { *errno_location };
                Err(nix::errno::Errno::from_i32(errno))
            }
        }
    }
}

pub struct PidFd(RawFd);

pub trait CreatePidFd {
    // mut isn't necessary here, but it is for the nightly PidFd stuff,
    // so keep it like this to avoid a warning when compiling using stable.
    fn create_pidfd(&mut self) -> nix::Result<PidFd>;
}

impl CreatePidFd for Child {
    fn create_pidfd(&mut self) -> nix::Result<PidFd> {
        pidfd_open(self.id() as RawFd)
    }
}

impl AsRawFd for PidFd {
    fn as_raw_fd(&self) -> RawFd {
        self.0
    }
}

impl Drop for PidFd {
    fn drop(&mut self) {
        let _ = nix::unistd::close(self.0);
    }
}
